#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <shellapi.h>
#include <stdio.h>
#include <stdlib.h>

#include "resource.h"

DISPLAY_DEVICE GetPrimaryDevice()
{
    int index=0;
    DISPLAY_DEVICE dd;
    dd.cb = sizeof(DISPLAY_DEVICE);

    while (EnumDisplayDevices(NULL, index++, &dd, 0))
    {
        if (dd.StateFlags & DISPLAY_DEVICE_PRIMARY_DEVICE) return dd;
    }
    return dd;
}

void SetDisplayDefaults()
{
    ChangeDisplaySettings(NULL, 0);
}

BOOL ToggleDisplayResolution(unsigned long PelsWidth, unsigned long PelsHeight)
{
    DISPLAY_DEVICE dd = GetPrimaryDevice();
    DEVMODE dm;
    char errbuf[512];
    dm.dmSize = sizeof(DEVMODE);
    if (!EnumDisplaySettings(dd.DeviceName, ENUM_CURRENT_SETTINGS, &dm))
    {
        snprintf(errbuf, sizeof(errbuf), "EnumDisplaySettings failed with error %d", (int)GetLastError());
		MessageBox(NULL, errbuf, NULL, MB_OK | MB_TOPMOST);
    }

    if (dm.dmPelsWidth == PelsWidth && dm.dmPelsHeight == PelsHeight)
    {
        SetDisplayDefaults();
        return TRUE;
    }

    dm.dmPelsWidth = PelsWidth;
    dm.dmPelsHeight = PelsHeight;
    dm.dmFields = (DM_PELSWIDTH | DM_PELSHEIGHT);
    if (ChangeDisplaySettings(&dm, CDS_TEST) !=DISP_CHANGE_SUCCESSFUL)
    {
        snprintf(errbuf, sizeof(errbuf), "Unsupported display mode %dx%d, error %d",
            (int)PelsWidth, (int)PelsHeight, (int)GetLastError());
		MessageBox(NULL, errbuf, NULL, MB_OK | MB_TOPMOST);
        return FALSE;
    }

    return (ChangeDisplaySettings(&dm, 0)==DISP_CHANGE_SUCCESSFUL);
}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
    LPWSTR *szArglist;
    int nArgs;
    szArglist = CommandLineToArgvW(GetCommandLineW(), &nArgs);
    if( NULL == szArglist )
    {
        MessageBox(NULL, "Failed parsing command line", NULL, MB_OK | MB_TOPMOST);
        return 1;
    }
    int x = 0;
    int y = 0;
    if (nArgs >= 2)
        x = _wtoi(szArglist[1]);
    if (nArgs >= 3)
        y = _wtoi(szArglist[2]);
    if (x==0)
        x = 800;
    if (y==0)
        y = 600;
    if (!ToggleDisplayResolution(x, y))
    {
        LocalFree(szArglist);
        return 2;
    }
    LocalFree(szArglist);
    return 0;
}
